########## mIRCStats Eggdrop Logger
###### (c)1999 JuLeS <jules01@chello.nl>  #manchester - IRCNet
###### Multiple Channel Support by SirSir <sirsir@thuntek.net>
######
###### This tcl should log channel activity the same way mIRC does
###### Logs made by this tcl are for use with the mIRCStats program

###### 12/03/00
######   Implemented addition written by Tric - When mEL backs up a
######   log file the date is added in the extension; handy to keep
######   track of your logfiles. Do note that older backups will NOT
######   be replaced by new backups (as it was in previous version of
######   mEL) This addition does not affect logfile handling when mEL
######   is in mAC mode. - Jules
###### 13/02/00
######   Completely rewrote the file handling. mEL now works on win32
######   systems as well. The script is optimized since it no longer
######   uses external executables for file handling (cp,mv,rm etc etc)
######   and thus it works on ported eggdrops as well. I've tested it
######   on eggdrop1.3.14 on win2000. The only odd thing is that I had
######   to set mEL to 1.1.5 logging to avoid problems catching mode
######   changes. But that's not a script error, it's prolly cos of 
######   the ported eggdrop. - Jules
###### 22/11/99
######   Well here is a version that will run on the 1.4.x version of
######   the eggdrop bot.  They changed some of the tcl commands so I
######   rewrote the script accordingly... - SirSir
######
###### Most recent updates since last release:
######   - Changed the TIME command for the 1.4.x eggdrop
######   - mEL now strips control codes - code provided by IRN-BRU
######   - mEL now runs on either 1.1.5 or 1.3.x/1.4.x eggdrops
######
###### Features as of v1.5
######   - Users can now put themselves on the mEL ignore list using a /msg
######     command.
######   - Changed the layout of the tcl a bit; also changed the partyline
######     command from .ircstats to .mel... 
######   - So-called 'logkeepers' are in control of mEL. There is one static
######     logkeeper who can assign other logkeepers.
######   - Users can now be ignored by mEL. Some people value their privacy
######     and I respect that...
######   - Logfiles of channels that aren't being logged anymore can be
######     erased from the partyline.
######
###### Features as of v1.4
######   - Multiple Channel Logging in the style of the commonly used irc
######     client mIRC.
######   - Variable amount of days to log a channel before closing the log-
######     file and starting a new one.
######   - Ability to turn mEL on or off from the partyline.
######   - mEL can be restarted from the partyline.
######   - Easily check mEL's status from the partyline.
######   - Add and remove channels to log from the partyline.
######   - Erase redundant logfiles easily.
######   - Add or remove extra logkeepers.
######   - Allow users to be excluded from channel logging.
######
###### Bug Fixes since the first version
######   - Added a separate directory for mEL to store its files in - Jules
######   - Fixed bug that occured when user tries to -chan the last channel
######     being logged - Jules
######   - Fixed bug where logfiles didn't get a proper begin nor end - Jules
######   - Fixed bug that didn't reset the variable 'currentday' - SirSir
######   - Fixed bug that overwrote the log file when ending the log - SirSir
######   - Fixed bug where mEL ignored users at random; problem appeared to be
######     a case-sensitive one - Jules
######   - Translated all procs into english; they were all in dutch! - Jules
######
###### Known Bugs
######   - mEL can't exclude users who don't exist in eggdrop, simply because
######     one handle could have several nicks, so it's faster to match handles
######
###### Many thanks to SirSir for the major update to multiple channel support!
###### SirSir practically rewrote most of the script to make it support
###### multiple channels!
######
###### Installation and usage
######   Simply fill in a proper path in the two variables listed below. It's
######   recommended, though, not to change these. If you do change them then
######   make sure they direct to a valid path!! "" is valid as well.. just
######   don't leave it blank. Do change the variable 'logkeeper' to suit to
######   your needs. This is the static logkeeper with full mEL control.
######   All other options are set from the partyline. I haven't included a
######   help file or function. If it doesn't seem to work and you don't get
######   real error messages then think logically and try it again.
######   If you do encounter serious problems then feel free to mail Jules at
######   jules01@chello.nl or SirSir at sirsir@thuntek.net

### These variables MUST be filled in correctly!!! Failure to do so will result
### in a non-working tcl!!! Mind the caps! :-P
###
### The paths MUST be valid BEFORE starting the bot!
### IE: Make sure the directories are there before you start the bot!

set statsdir "logs/"
set statslogdir "logs/"
set logkeeper "lando"

### Variable melegg makes this tcl work on 1.1.5, 1.3.x and 1.4.x
### Commenting out the variable or leaving it blank will make mEL default
### to eggdrop 1.3.x and 1.4.x
### Note: On ported eggdrops sometimes you get an error while logging. Try
### setting melegg to an older version of eggdrop then.
### (1 = 1.1.5  2 = 1.3.x  3 = 1.4.x)

set melegg 3

# # # # # # # # # # # # DO NOT CHANGE ANYTHING BELOW HERE # # # # # # # # # # #

set logver "v1.5.2"
set vername "\002mEL $logver:\002"
set daystolog 5
set currentday 1
set switch 1
set switched 0
set checktimer "00:00"
set statschanfile "channels.mel"
set keeperfile "keepers.mel"
set ignorefile "dontlog.mel"
set daysfile "days.mel"
set logdayfile "currentday.mel"
set removedchannels "rmchans.mel"
set macfile "checkmac.mel"

bind dcc n mel dcc_mel
bind msg - "!mel" msg_mel
bind pubm - * chatter
bind join - * enter
bind part - * parting
bind sign - * signoff
bind topc - * ctopic
bind kick - * kicked
bind nick - * nickchange
if {$melegg == 1} {
	bind mode - * cmodea
} else {
	bind mode - * cmode
}
bind ctcp - "ACTION" caction

proc dcc_mel {handle idx arg} {
	global statsdir logkeeper statschanfile switch daystolog autoswitch currentday daysfile daystolog logdayfile logver keeperfile vername ignorefile macfile
	set cmd [lindex $arg 0]
	set cmd2 [lindex $arg 1]
	if {![findkeeper [string tolower $handle]]} {
		putidx $idx "You are not allowed to perform maintenance!"
		putidx $idx "Contact $logkeeper for more information"
		putidx $idx "\002mIRCStats Eggdrop Logger\002 $logver"
		return 0
	}
	if {$cmd == "on" } {
		putlog "#$handle# mel on"
		if {$switch == 1} {
			putidx $idx "mEL logger already is on!"
			return 0
		}
		if {![file exists ${statsdir}${statschanfile}]} {
			putidx $idx "You must use .mel +chan <channel> first!"
			return 0
		}
		set switch 1
		putidx $idx "mEL logger turned on!"
		return 0
	}
	if {$cmd == "off"} {
		putlog "#$handle# mel off"
		if {$switch == 0} {
			putidx $idx "mEL logger already turned off!"
			return 0
		}
		set switch 0
		putidx $idx "mEL logger turned off!"
		return 0
	}
	if {$cmd == "restart"} {
		putlog "#$handle# mel restart"
		if {$switch == 0} {
			putidx $idx "$vername Can't restart logfile -> Logging is disabled"
			return 0
		}
		if {![file exists ${statsdir}${statschanfile}]} {
			putidx $idx "$vername You must use .mel +chan <channel> first!"
			return 0
		}
		if {[checkmacfile]} {
			putidx $idx "$vername You can't restart manually when in mAC mode!"
			return 0
		}
		putlog "$vername Restarting logging at day one!"
		set autoswitch 0
		switchlogs
		return 0
	}
	if {$cmd == "setdays"} {
		putlog "#$handle# mel setdays"
		if {$switch == 0} {
			putidx $idx "$vername Can't change amount of days to log -> Logging is disabled!"
			return 0
		}
		if {$cmd2 == "0"} {
			if {[checkmacfile]} {
				putidx $idx "$vername mEL is already running in mAC mode!"
				return 0
			}
			putidx $idx "Switching to mAC mode..."
			foreach killt [timers] {
				if {[lindex $killt 1] == "logcheck"} {
					killtimer [lindex $killt 2]
				}
			}
			set create [open ${statsdir}${macfile} w]
			puts $create "mAC mode check file - DO NOT REMOVE!!!"
			close $create
			set autoswitch 1
			switchlogs
			putidx $idx "$vername mEL has switched to mAC mode. Switching back to manual mode will restart logging!"
			return 0
		}
		if {$cmd2 != "0"} {
			if {![checkmacfile]} {
				set daystolog $cmd2
				putidx $idx "$vername Changing logging to $cmd2 days"
				if {[file exists ${statsdir}${daysfile}]} {
					set remove [file delete ${statsdir}${daysfile}]
				}
				set create [open ${statsdir}${daysfile} w]
				puts $create "$daystolog"
				close $create
				return 0
			} else {
				putidx $idx "$vername Switching back to manual mode..."
				set remove [file delete ${statsdir}${macfile}]
				timer 1 logcheck
				set currentday 1
				if {[file exists ${statsdir}${logdayfile}]} {
					set remove [file delete ${statsdir}${logdayfile}]
				}
				set create [open ${statsdir}${logdayfile} w]
				puts $create "$currentday"
				close $create
				set daystolog $cmd2
				if {[file exists ${statsdir}${daysfile}]} {
					set remove [file delete ${statsdir}${daysfile}]
				}
				set create [open ${statsdir}${daysfile} w]
				puts $create "$daystolog"
				close $create
				set autoswitch 1
				switchlogs
				putidx $idx "$vername Back in manual mode."
				return 0
			}
			return 0
		}
		return 0
	}
	if {$cmd == "status"} {
		putlog "#$handle# mel status"
		if {$switch == 0} {
			putidx $idx "mEL MUST be turned on first."
			return 0
		}
		readdays
		set status "Off"
		if {$switch == 1} {
			set status "On"
		}
		putidx $idx "mEL is turned $status"
		if {[checkmacfile]} {
			putidx $idx "mEL is running in mAC mode."
		} else {
			putidx $idx "mEL is logging on day $currentday of $daystolog day(s)"
		}
		set read [open ${statsdir}${statschanfile} r]
		while {![eof $read]} {
			set data [string tolower [gets $read]]
			if {[eof $read]} {break}
			if {$data != ""} {
				putidx $idx "	 Logging $data"
			}
		}
		close $read
		return 0
	}
	if {$cmd == "+chan" && $cmd2 != ""} {
		putlog "#$handle# mel +chan $cmd2"
		if {[findchan $cmd2]} {
			putidx $idx "$vername $cmd2 is already being logged!"
			return 0
		}
		set temp [addlogchan $cmd2]
		putidx $idx "mEL is logging $cmd2"
		return 0
	}
	if {$cmd == "-chan" && $cmd2 != ""} {
		putlog "#$handle# mel -chan $cmd2"
		if {![file exists ${statsdir}${statschanfile}]} {
			putidx $idx "$vername You must use .mel +chan <channel> first!"
			return 0
		}
		if {![findchan $cmd2]} {
			putidx $idx "$vername $cmd2 is not being logged!"
			return 0
		}
		set temp [rmlogchan $cmd2]
		putidx $idx "mEL is no longer logging $cmd2"
		putidx $idx "Use .mel erase to remove the logfile!"
		return 0
	}
	if {$cmd == "erase"} {
		putlog "#$handle# mel erase"
		killog $handle
		return 0
	}
	if {$cmd == "addkeeper" && $cmd2 != ""} {
		putlog "#$handle# mel addkeeper $cmd2"
		if {[string tolower $handle] != [string tolower $logkeeper]} {
			putidx $idx "$vername Sorry, you can't add logkeepers!"
			return 0
		}
		if {![validuser $cmd2]} {
			putidx $idx "$vername This is not a valid handle!"
			return 0
		}
		if {[findkeeper $cmd2]} {
			putidx $idx "$vername $cmd2 already is a logkeeper!"
			return 0
		}
		set temp [addkeeper $cmd2]
		putidx $idx "$vername User added successfully!"
		return 0
	}
	if {$cmd == "rmkeeper" && $cmd2 != ""} {
		putlog "#$handle# mel rmkeeper $cmd2"
		if {[string tolower $handle] != [string tolower $logkeeper]} {
			putidx $idx "$vername Sorry, you can't remove logkeepers!"
			return 0
		}
		if {![findkeeper $cmd2]} {
			putidx $idx "$vername $cmd2 isn't a logkeeper to begin with!"
			return 0
		}
		if {[string tolower $logkeeper] == $cmd2} {
			putidx $idx "$vername Unable to remove static logkeeper!"
			return 0
		}
		set temp [rmkeeper $cmd2]
		putidx $idx "$vername User removed successfully!"
		return 0
	}
	if {$cmd == "listkeeper"} {
		putlog "#$handle# mel listkeeper"
		putidx $idx "The following handles have logkeeper status:"
		if {[file exists ${statsdir}${keeperfile}]} {
			set read [open ${statsdir}${keeperfile} r]
			while {![eof $read]} {
				set data [string tolower [gets $read]]
				if {[eof $read]} {break}
				if {$data != ""} {
					putidx $idx "   $data"
				}
			}
			close $read
		}
		return 0
	}
	if {$cmd == "addignore" && $cmd2 != ""} {
		putlog "#$handle# mel addignore $cmd2"
		if {![validuser $cmd2]} {
			putidx $idx "$vername $cmd2 is not a valid user!"
			return 0
		}
		if {[checkignore $cmd2]} {
			putidx $idx "$vername $cmd2 already is on the ignore list!"
			return 0
		}
		set temp [addign $cmd2]
		putidx $idx "$vername No longer logging public chatter of $cmd2!"
		return 0
	}
	if {$cmd == "unignore" && $cmd2 != ""} {
		putlog "#$handle# mel unignore $cmd2"
		if {![checkignore $cmd2]} {
			putidx $idx "$vername $cmd2 was not found in the ignore list!"
			return 0
		}
		set temp [unign $cmd2]
		putidx $idx "$vername Removed $cmd2 from the ignore list!"
		return 0
	}
	if {$cmd == "ignores"} {
		putlog "#$handle# mel ignores"
		putidx $idx "Users currently in ignore list:"
		if {![file exists ${statsdir}${ignorefile}]} {
			putidx $idx "  None"
			return 0
		}
		if {[file exists ${statsdir}${ignorefile}]} {
			set read [open ${statsdir}${ignorefile} r]
			while {![eof $read]} {
				set data [string tolower [gets $read]]
				if {[eof $read]} {break}
				putidx $idx "  $data"
			}
			close $read
		}
		return 0
	}
	putidx $idx "\002Usage:\002 .mel <on/off/restart/setdays/status/+chan/-chan/erase/addkeeper/rmkeeper/listkeeper/addignore/unignore/ignores>"
}

proc msg_mel {nick uhost handle arg} {
	global statsdir switch logver vername ignorefile botnick
	set cmd [lindex $arg 0]
	if {![validuser $handle]} {
		putlog "$vername mel $cmd - User not recognised -> Ignoring"
		return 0
	}
	putlog "\[$nick!$uhost\] mel $cmd"
	if {$cmd == "ignoreme"} {
		set temp [addign $handle]
		putserv "NOTICE $nick : $vername From now on you will be ignored by mEL!"
		return 0
	}
	if {$cmd == "unignoreme"} {
		set temp [unign $handle]
		putserv "NOTICE $nick : $vername You are no longer being ignored by mEL!"
		return 0
	}
	if {$cmd == "ignorestatus"} {
		if {[checkignore $handle]} {
			putserv "NOTICE $nick : $vername You are currently being ignored by mEL!"
			return 0
		}
		putserv "NOTICE $nick : $vername You are currently not being ignored by mEL!"
		return 0
	}
	putserv "NOTICE $nick : $vername Usage: /msg $botnick mel <ignoreme/unignoreme/ignorestatus> <password>"
	return 0
}

proc startlog {} {
	global statschanfile statslogdir switch daystolog autoswitch daysfile logdayfile currentday statsdir vername botnick
	set currenttime [unixtime]
	set starttime [ctime $currenttime]
	set current [strftime %H:%M]
	if {![file exists ${statsdir}${statschanfile}]} {
		if {![file exists ${statsdir}${statschanfile}.bak]} {
			putlog "$vername ${statsdir}${statschanfile} is missing."
			putlog "$vername Use .mel +chan <channel>."
			set switch 0
			return 0
		}
		set copytonew [file copy -force ${statsdir}${statschanfile}.bak ${statsdir}${statschanfile}]
		set removeold [file delete -force ${statsdir}${statschanfile}.bak]
	}
	set read [open ${statsdir}${statschanfile} r]
	while {![eof $read]} {
		set data [gets $read]
		if {[eof $read]} {break}
		if {![file exists ${statslogdir}${data}.log]} {
			set create [open ${statslogdir}${data}.log w]
			puts $create "Session Start: $starttime"
			puts $create "\[$current\] *** Now talking in $data"
			puts $create "\[$current\] <$botnick> Hi"
			close $create
			putlog "$vername Created logfile for ${statslogdir}${data}.log"
		}
	}
	close $read
	if {![file exists ${statsdir}${daysfile}]} {
		set create [open ${statsdir}${daysfile} w]
		puts $create "$daystolog"
		close $create
	}
	if {![file exists ${statsdir}${logdayfile}]} {
		set create [open ${statsdir}${logdayfile} w]
		puts $create "$currentday"
		close $create
	}
}

proc checkmacfile {} {
	global statsdir macfile
	if {[file exists ${statsdir}${macfile}]} {
		return 1
	} else {
		return 0
	}
}

proc createlog {channel} {
	global statschanfile statslogdir switch statsdir vername botnick
	set starttime [ctime [unixtime]]
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {![findchan $chan]} {
		putlog "$vername Error! Tried to create new logfile for $chan but couldn't verify with $statschanfile"
		return 0
	}
	if {![file exists ${statslogdir}${chan}.log]} {
		set create [open ${statslogdir}${chan}.log w]
		puts $create "Session Start: $starttime"
		puts $create "\[$current\] *** Now talking in $chan"
		puts $create "\[$current\] <$botnick> Hi"
		close $create
		putlog "$vername Created logfile for ${statslogdir}${chan}.log"
	}
	if {$switch == 0} {
		set switch 1
		putlog "$vername Logger turned on!"
	}
	return 0
}

proc endlog {channel} {
	global statschanfile statslogdir switch statsdir vername
	set endtime [ctime [unixtime]]
	set chan [string tolower $channel]
	if {![findchan $chan]} {
		putlog "$vername Error! Tried to close log of channel not being logged according to $statschanfile"
		return 0
	}
	if {![file exists ${statslogdir}${chan}.log]} {
		putlog "$vername Error! Tried to close non-existant logfile!"
		return 0
	}
	set add "Session Close: $endtime"
	set addfile [open ${statslogdir}${chan}.log a]
	puts $addfile $add
	close $addfile
	putlog "$vername Closed logfile for channel $chan"
	return 0
}

proc rmlogchan {channel} {
	global statschanfile switch removedchannels statsdir vername
	set chan [string tolower $channel]
	endlog $chan
	set copyfile [file copy -force ${statsdir}${statschanfile} ${statsdir}${statschanfile}.bak]
	set read [open ${statsdir}${statschanfile}.bak r]
	while {![eof $read]} {
		set data [string tolower [gets $read]]
		if {[eof $read]} {break}
		if {$chan != $data && $data != ""} {
			set addfile [open ${statsdir}${statschanfile}.new a]
			puts $addfile $data
			close $addfile
		}
	}
	close $read
	if {![file exists ${statsdir}$removedchannels]} {
		set create [open ${statsdir}${removedchannels} w]
		puts $create $chan
		close $create
	}
	if {[file exists ${statsdir}${removedchannels}]} {
		set addfile [open ${statsdir}${removedchannels} a]
		puts $addfile $chan
		close $addfile
	}
	if {![file exists ${statsdir}${statschanfile}.new]} {
		set removefile [file delete -force ${statsdir}${statschanfile}]
		putlog "$vername No more channels to be logged! Shutting down logger!"
		set switch 0
	}
	if {[file exists ${statsdir}${statschanfile}.new]} {
		set copytonew [file copy -force ${statsdir}${statschanfile}.new ${statsdir}${statschanfile}]
		set removeold [file delete -force ${statsdir}${statschanfile}.new]
	}
	set removefile [file delete -force ${statsdir}${statschanfile}.bak]
	return 1
}

proc switchlogs {} {
	global statschanfile statslogdir logdayfile daysfile daystolog currentday switch autoswitch switched statsdir vername
	if {$switch == 0} {return 0}
	set read [open ${statsdir}${statschanfile} r]
	while {![eof $read]} {
		set data [gets $read]
		if {[eof $read]} {break}
		set switched 1
		set closetime [ctime [unixtime]]
		set add "Session Close: $closetime"
		set addfile [open ${statslogdir}${data}.log a]
		puts $addfile $add
		close $addfile
		set secs [clock seconds]
		set cdate [clock format [incr secs -61] -format %m.%d.%Y]
		set copynew [file copy -force ${statslogdir}${data}.log ${statslogdir}${data}.${cdate}.log]
		set removeold [file delete -force ${statslogdir}${data}.log]
		putlog "$vername Created a backup copy of ${statslogdir}${data}.log to ${statslogdir}${data}.${cdate}.log file after $currentday day(s)."
	}
	close $read
	set remove [file delete -force ${statsdir}${logdayfile}]
	set currentday 1
	putlog "$vername Reset current logday to day one"
	if {$autoswitch == 1} {
		putlog "$vername Procedure called automatically"
	}
	if {$autoswitch == 0} {
		putlog "$vername Procedure called upon users request"
	}
	startlog
}     

proc killog {handle} {
	global removedchannels statslogdir statsdir
	set idx [hand2idx $handle]
	if {![file exists ${statsdir}${removedchannels}]} {
		putidx $idx "All closed logfiles are already erased"
		return 0
	}
	putidx $idx "Removing closed logfiles..."
	set read [open ${statsdir}${removedchannels} r]
	while {![eof $read]} {
		set data [gets $read]
		if {[eof $read]} {break}
		if {[file exists ${statslogdir}${data}.log]} {
			set remove [file delete -force ${statslogdir}${data}.log]
			putidx $idx "	Removed ${statslogdir}${data}.log..."
		}
		if {[file exists ${statslogdir}${data}.log~bak]} {
			set remove [file delete -force ${statslogdir}${data}.log~bak]
			putidx $idx "	Removed ${statslogdir}${data}.log~bak..."
		}
	}
	close $read
	set remove [file delete -force ${statsdir}${removedchannels}]
	putidx $idx "Removed all closed logfiles..."
	return 0
}

proc addlogchan {channel} {
	global statschanfile statsdir
	set chan [string tolower $channel]
	if {[file exists ${statsdir}${statschanfile}]} {
		set addfile [open ${statsdir}${statschanfile} a]
		puts $addfile $chan
		close $addfile
		createlog $chan
		return 1
	}
	set create [open ${statsdir}${statschanfile} w]
	puts $create $chan
	close $create
	createlog $chan
	return 1
}

proc findchan {channel} {
	global statschanfile statsdir
	set chan [string tolower $channel]
	if {[file exists ${statsdir}${statschanfile}]} {
		set read [open ${statsdir}${statschanfile} r]
		while {![eof $read]} {
			set data [string tolower [gets $read]]
			if {[eof $read]} {break}
			if {$chan == $data && $data != ""} {
				close $read
				return 1
			}
		}
		close $read
	}
	return 0
}

proc findkeeper {keeper} {
	global keeperfile logkeeper statsdir
	set keep [string tolower $keeper]
	if {$keep == [string tolower $logkeeper]} {return 1}
	if {[file exists ${statsdir}${keeperfile}]} {
		set read [open ${statsdir}${keeperfile} r]
		while {![eof $read]} {
			set data [string tolower [gets $read]]
			if {[eof $read]} {break}
			if {$keep == $data && $data != ""} {
				close $read
				return 1
			}
		}
		close $read
	}
	return 0
}

proc addkeeper {keeper} {
	global keeperfile logkeeper statsdir vername
	set keep [string tolower $keeper]
	if {![file exists ${statsdir}${keeperfile}]} {
		set create [open ${statsdir}${keeperfile} w]
		puts $create $keep
		close $create
		putlog "$vername New logkeeper added!"
		return 0
	}
	if {[file exists ${statsdir}${keeperfile}]} {
		set addfile [open ${statsdir}${keeperfile} a]
		puts $addfile $keep
		close $addfile
		putlog "$vername New logkeeper added!"
		return 0
	}
	return 0
}

proc rmkeeper {keeper} {
	global keeperfile logkeeper statsdir vername
	set keep [string tolower $keeper]
	if {![file exists ${statsdir}${keeperfile}]} {
		putlog "$vername Error! Unable to locate $keeperfile - Creating new file"
		createkeepfile
		return 0
	}
	if {[file exists ${statsdir}${keeperfile}]} {
		set read [open ${statsdir}${keeperfile} r]
		while {![eof $read]} {
			set data [string tolower [gets $read]]
			if {[eof $read]} {break}
			if {$keep != $data && $data !=""} {
				set addfile [open ${statsdir}${keeperfile}.new a]
				puts $addfile $data
				close $addfile
			}
		}
		close $read
		if {![file exists ${statsdir}${keeperfile}.new]} {
			putlog "$vername Error! $keeperfile found empty! - Creating new file"
			createkeepfile
			return 0
		}
		set removefile [file delete -force ${statsdir}${keeperfile}]
		set copynew [file copy -force ${statsdir}${keeperfile}.new ${statsdir}${keeperfile}]
		set removeold [file delete -force ${statsdir}${keeperfile}.new]
	}
	return 0
}

proc createkeeper {} {
	global keeperfile logkeeper statsdir vername
	if {![file exists ${statsdir}${keeperfile}]} {
		set create [open ${statsdir}${keeperfile} w]
		puts $create $logkeeper
		close $create
		putlog "$vername Logkeepers identified..."
		return 0
	}
	return 0
}

proc addign {name} {
	global statsdir ignorefile vername
	set ign [string tolower $name]
	if {![file exists ${statsdir}${ignorefile}]} {
		set create [open ${statsdir}${ignorefile} w]
		puts $create $ign
		close $create
		putlog "$vername Added user to ignore list..."
		return 0
	}
	set addfile [open ${statsdir}${ignorefile} a]
	puts $addfile $ign
	close $addfile
	putlog "$vername Added user to ignore list..."
	return 0
}

proc unign {name} {
	global statsdir ignorefile vername
	set ign [string tolower $name]
	set copyfile [file copy -force ${statsdir}${ignorefile} ${statsdir}${ignorefile}.bak]
	set read [open ${statsdir}${ignorefile}.bak r]
	while {![eof $read]} {
		set data [string tolower [gets $read]]
		if {[eof $read]} {break}
		if {$ign != $data && $data != ""} {
			set addfile [open ${statsdir}${ignorefile}.new a]
			puts $addfile $data
			close $addfile
		}
	}
	close $read
	if {![file exists ${statsdir}${ignorefile}.new]} {
		set removefile [file delete -force ${statsdir}${ignorefile}.new]
		putlog "$vername Removed last ignored user!"
		return 0
	}
	set remove [file delete -force ${statsdir}${ignorefile}]
	set remove [file delete -force ${statsdir}${ignorefile}.bak]
	set copynew [file copy -force ${statsdir}${ignorefile}.new ${statsdir}${ignorefile}]
	set removeold [file delete -force ${statsdir}${ignorefile}.new]
	putlog "$vername Removed single user from ignore list!"
	return 0
}

proc checklogrestart {channel} {
	global statslogdir
	set chan [string tolower $channel]
	if {![file exists ${statslogdir}${chan}.log]} {
		createlog $chan
	}
}

proc checkignore {name} {
	global statsdir ignorefile vername
	set ign [string tolower $name]
	if {[file exists ${statsdir}${ignorefile}]} {
		set read [open ${statsdir}${ignorefile} r]
		while {![eof $read]} {
			set data [string tolower [gets $read]]
			if {[eof $read]} {break}
			if {$ign == $data && $data != ""} {
				close $read
				return 1
			}
		}
		close $read
	}
	return 0
}

proc readdays {} {
	global daysfile daystolog logdayfile currentday statsdir
	if {[file exists ${statsdir}${daysfile}]} {
		set read [open ${statsdir}${daysfile} r]
		while {![eof $read]} {
			set days [gets $read]
			if {[eof $read]} {break}
			set daystolog [lindex $days 0]
		}
		close $read
	}
	if {[file exists ${statsdir}${logdayfile}]} {
		set read [open ${statsdir}${logdayfile} r]
		while {![eof $read]} {
			set days [gets $read]
			if {[eof $read]} {break}
			set currentday [lindex $days 0]
		}
		close $read
	}
}

proc checkdays {} {
	global logdayfile daysfile daystolog currentday switch autoswitch switched statsdir vername
	set switched 0
	if {$switch == 0} {return 0}
	readdays
	if {$currentday > $daystolog} {
		set autoswitch 1
		switchlogs
		set switched 1
		return 0
	}
	if {$switched == 0} {
		incr currentday
		if {[file exists ${statsdir}${logdayfile}]} {
			set remove [file delete -force ${statsdir}${logdayfile}]
		}
	set create [open ${statsdir}${logdayfile} w]
	puts $create "$currentday"
	close $create
	}
}

proc logcheck {} {
	global checktimer switch
	if {$switch == 0} {return 0}
	set current [strftime %H:%M]
	if {$current == $checktimer} {checkdays}
	foreach killt [timers] {
		if {[lindex $killt 1] == "logcheck"} {
			killtimer [lindex $killt 2]
		}
	}
	timer 1 logcheck
}

## strip colour codes from text
proc strip {args} {
    regsub -all \{|\} $args "" args
    regsub -all \002 $args "" args
    regsub -all \037 $args "" args
    regsub -all  $args "" args
    regsub -all  $args "" args
    regsub -all {(([0-9])?([0-9])?(\,([0-9])?([0-9])?)?)?} $args "" args
    regsub -all {([0-9A-F][0-9A-F])?} $args "" args
    ## since all the colour codes have gone, trim any trailing space we may have.
    set arg [string trimleft $args]
    return $args
}

proc chatter {nick host handle channel text} {
	global statslogdir switch
	if {$switch == 0 } { return 0 }
	set name $nick
	set phrase [strip $text]
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] <$name> $phrase"
		if {[checkignore $handle]} {
			return 0
		}
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc enter {nick host handle channel} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set uhost $host
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name ($uhost) has joined $chan"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc parting {nick host handle channel} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set uhost $host
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name ($uhost) has left $chan"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc signoff {nick host handle channel reason} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set quitmsg [strip $reason]
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name has left IRC ($quitmsg)"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc ctopic {nick host handle channel topic} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set newtopic [strip $topic]
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name changes topic to '$newtopic'"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc kicked {nick host handle channel target reason} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set loser $target
	set why [strip $reason]
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $loser was kicked by $name ($why)"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc nickchange {nick host handle channel newnick} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set newname $newnick
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name is now known as $newname"
		if {[checkignore $handle]} {
			return 0
		}
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc cmode {nick host handle channel change victim} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set newmode $change
	set lewser $victim
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name sets mode: $newmode $lewser"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc cmodea {nick host handle channel change} {
	global statslogdir switch
	if {$switch == 0} {return 0}
	set name $nick
	set newmode $change
	set current [strftime %H:%M]
	set chan [string tolower $channel]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set add "\[$current\] *** $name sets mode: $newmode"
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

proc caction {nick host handle dest keyword arg} {
	global switch statslogdir
	if {$switch == 0} {return 0}
	set name $nick
	set current [strftime %H:%M]
	set chan [string tolower $dest]
	if {[findchan $chan]} {
		if {[checkmacfile]} {
			checklogrestart $chan
		}
		set doing [strip $arg]
		set add "\[$current\] * $name $doing"
		if {[checkignore $handle]} {
			return 0
		}
		set adding [open ${statslogdir}${chan}.log a]
		puts $adding "$add"
		close $adding
	}
}

readdays
startlog
createkeeper
if {![file exists ${statsdir}${macfile}]} {
	timer 1 logcheck
}

if {$melegg == 1} {
	set melrun "1.1.5"
} else {
	set melrun "1.3.x and 1.4.x"
}

putlog "\002mIRCStats Eggdrop Logger (mEL)\002 $logver by Jules loaded!"
putlog "\002mIRCStats Eggdrop Logger (mEL)\002 $logver Multi-Channel Edition by SirSir loaded!"
putlog "\002mIRCStats Eggdrop Logger (mEL)\002 running in eggdrop $melrun mode."

### http://www.xs4all.nl/~sjuul/tcl/index.html
### Feel free to alter this tcl to your personal flavour... just don't forget who wrote the
### original code... Also, if you find bugs, or even fix some.. then please let me know!
